﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;

namespace howto_crop_to_size_scaled
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private Point ULCorner = new Point(0, 0);

        private bool Dragging = false;
        private Point LastPoint;

        private Bitmap OriginalImage = null;
        private Bitmap ScaledImage = null;

        private float ImageScale = 1f;

        // Let the user select an image file.
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Load the image.
                    OriginalImage = LoadBitmapUnlocked(ofdImage.FileName);
                    ShowScaledImage();

                    // Enable the Save As menu item.
                    mnuFileSaveAs.Enabled = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        // Display the scaled image.
        private void ShowScaledImage()
        {
            if (OriginalImage == null) return;
            int scaled_width = (int)(OriginalImage.Width * ImageScale);
            int scaled_height = (int)(OriginalImage.Height * ImageScale);
            ScaledImage = new Bitmap(scaled_width, scaled_height);
            using (Graphics gr = Graphics.FromImage(ScaledImage))
            {
                Point[] dest_points =
                {
                    new Point(0, 0),
                    new Point(scaled_width - 1, 0),
                    new Point(0, scaled_height - 1),
                };
                Rectangle src_rect = new Rectangle(
                    0, 0,
                    OriginalImage.Width - 1,
                    OriginalImage.Height - 1);
                gr.DrawImage(OriginalImage, dest_points, src_rect, GraphicsUnit.Pixel);
            }
            picImage.Image = ScaledImage;
            picImage.Visible = true;
            picImage.Refresh();
        }

        // Save the selected area.
        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    // Copy the selected area into a new Bitmap.
                    Rectangle rect = SelectionRectangle(false);
                    Bitmap bm = new Bitmap(rect.Width, rect.Height);
                    using (Graphics gr = Graphics.FromImage(bm))
                    {
                        gr.DrawImage(OriginalImage, 0, 0, rect,
                            GraphicsUnit.Pixel);
                    }

                    // Save the new Bitmap.
                    SaveImage(bm, sfdImage.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Draw the selection rectangle.
        private void picImage_Paint(object sender, PaintEventArgs e)
        {
            try
            {
                // Draw the selection rectangle.
                using (Pen pen = new Pen(Color.Red, 2))
                {
                    Rectangle rect = SelectionRectangle(true);
                    e.Graphics.DrawRectangle(pen, rect);

                    pen.Color = Color.Yellow;
                    pen.DashPattern = new float[] { 5, 5 };
                    e.Graphics.DrawRectangle(pen, rect);
                }
            }
            catch
            {
            }
        }

        // Return the currently selected area.
        private Rectangle SelectionRectangle(bool scaled)
        {
            int x, y, width, height;

            // Get the desired dimensions.
            // If there's a problem, return a zero-size rectangle.
            if (!int.TryParse(txtWidth.Text, out width))
                return new Rectangle();
            if (!int.TryParse(txtHeight.Text, out height))
                return new Rectangle();
            x = ULCorner.X;
            y = ULCorner.Y;

            // Return the rectangle.
            if (scaled)
            {
                x = (int)(x * ImageScale);
                y = (int)(y * ImageScale);
                width = (int)(width * ImageScale);
                height = (int)(height * ImageScale);
            }
            return new Rectangle(x, y, width, height);
        }

        // Start dragging.
        private void picImage_MouseDown(object sender, MouseEventArgs e)
        {
            // If the mouse is not inside the
            // selection rectangle, do nothing.
            Rectangle rect = SelectionRectangle(true);
            if (!rect.Contains(e.Location)) return;

            // Start the drag.
            LastPoint = e.Location;
            Dragging = true;
        }

        // Continue dragging.
        private void picImage_MouseMove(object sender, MouseEventArgs e)
        {
            if (!Dragging) return;
            ULCorner.X += (int)((e.Location.X - LastPoint.X) / ImageScale);
            ULCorner.Y += (int)((e.Location.Y - LastPoint.Y) / ImageScale);
            LastPoint = e.Location;
            picImage.Refresh();
        }

        // Stop dragging.
        private void picImage_MouseUp(object sender, MouseEventArgs e)
        {
            Dragging = false;
        }

        private void txt_TextChanged(object sender, EventArgs e)
        {
            picImage.Refresh();
        }

        private void mnuScale_Click(object sender, EventArgs e)
        {
            // Get the scale factor.
            ToolStripMenuItem menu_item =
                sender as ToolStripMenuItem;
            string scale_text = menu_item.Text.Replace("&", "").Replace("%", "");
            ImageScale = float.Parse(scale_text) / 100f;
            ShowScaledImage();

            // Display the new scale.
            mnuScale.Text = "Scale (" + menu_item.Text.Replace("&", "") + ")";

            // Check the selected menu item.
            foreach (ToolStripMenuItem item in mnuScale.DropDownItems)
            {
                item.Checked = (item == menu_item);
            }
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }
    }
}
